@@ -7,6 +7,10 @@ import json
 from datetime import datetime, timedelta
 from typing import Union
 import logging
+import mimetypes
+import base64
+import os.path
+from io import IOBase
 
 logger = logging.getLogger()
 
@@ -29,6 +33,8 @@ def initService(obj: Union[str, dict]):
             OAuth2Service.from_dict,
             LoginService.from_json,
             LoginService.from_dict,
+            BaseService.from_json,
+            BaseService.from_dict
         ]
     )
     return load(obj)
@@ -43,8 +49,24 @@ class BaseService:
     _implements = None
     _fileTransferMode = None
     _fileTransferArchive = None
+    _description = None
+    _icon = None
+    _infoUrl = None
+    _helpUrl = None
+    _displayName = None
 
-    def __init__(self, servicename: str, implements: list = None, fileTransferMode: FileTransferMode = FileTransferMode.active, fileTransferArchive: FileTransferArchive = FileTransferArchive.none):
+    def __init__(
+        self,
+        servicename: str,
+        implements: list = None,
+        fileTransferMode: FileTransferMode = FileTransferMode.active,
+        fileTransferArchive: FileTransferArchive = FileTransferArchive.none,
+        description: dict = None,
+        icon: str = "",
+        infoUrl: str = "",
+        helpUrl: str = "",
+        displayName: str = None
+    ):
         """Initialize Service without any authentication.
 
         Args:
@@ -52,11 +74,35 @@ class BaseService:
             implements (list, optional): Specified the implemented port endpoints. Defaults to empty list.
             fileTransferMode (int, optional): Set the mode for transfering files. Defaults to 0=active. Alternative is 1=passive.
             fileTransferArchive (str, optional): Set the archive, which is needed for transfering files. Defaults to "". Other value is "zip"
+            description (dict, optional): Set a short description for this service with corresponding language. Defaults to {"en":""}.
+            icon: (str, optional): Takes a filepath, so the mimetype and base64 can be calculated for later usage. Defaults to "".
+            infoUrl: (str, optional): Set the infoUrl for this service, so the user can be redirected to it to find more information about the service. Defaults to "".
+            helpUrl: (str, optional): Set the helpUrl for this service, so the user can be redirected to a helpdesk page about this service. Defaults to "".
+            displayName: (str, optional): Set the displayName for this service, which can be different as the servicename. Servicename will be used for identifiers. Defaults to "".
         """
         self.check_string(servicename, "servicename")
 
         self._servicename = servicename.lower()
 
+        if description is None:
+            self._description = {"en": ""}
+        else:
+            self._description = description
+
+        self._infoUrl = infoUrl
+        self._helpUrl = helpUrl
+        self._displayName = displayName
+
+        if icon is not None and icon != "":
+            if os.path.isfile(icon):
+                mime = mimetypes.guess_type(icon)[0]
+
+                with open(icon, "rb") as f:
+                    b64 = base64.b64encode(f.read())
+                    self._icon = f"data:{mime};base64,{b64}"
+            else:
+                raise FileNotFoundError
+
         self._implements = implements
         if implements is None:
             self._implements = []
@@ -65,7 +111,9 @@ class BaseService:
 
         if len(self._implements) == 0 or len(self._implements) > 2:
             raise ValueError(
-                "implements is empty or over 2 elements. Value: {}, Only valid: {}".format(len(self._implements), valid_implements))
+                "implements is empty or over 2 elements. Value: {}, Only valid: {}".format(
+                    len(self._implements), valid_implements)
+            )
 
         for impl in self._implements:
             if impl not in valid_implements:
@@ -87,6 +135,26 @@ class BaseService:
     def fileTransferArchive(self):
         return self._fileTransferArchive
 
+    @property
+    def description(self):
+        return self._description
+
+    @property
+    def icon(self):
+        return self._icon
+
+    @property
+    def infoUrl(self):
+        return self._infoUrl
+
+    @property
+    def helpUrl(self):
+        return self._helpUrl
+
+    @property
+    def displayName(self):
+        return self._displayName
+
     @property
     def implements(self):
         return self._implements
@@ -127,7 +195,12 @@ class BaseService:
             "servicename": self._servicename,
             "implements": self._implements,
             "fileTransferMode": self.fileTransferMode.value,
-            "fileTransferArchive": self.fileTransferArchive.value
+            "fileTransferArchive": self.fileTransferArchive.value,
+            "description": self._description,
+            "icon": self._icon,
+            "infoUrl": self._infoUrl,
+            "helpUrl": self._helpUrl,
+            "displayName": self._displayName
         }
 
         return data
@@ -146,8 +219,20 @@ class BaseService:
 
         if "type" in data and str(data["type"]).endswith("Service") and "data" in data:
             data = data["data"]
-            if "servicename" in data:
-                return BaseService(data["servicename"], data.get("implements"), FileTransferMode(data.get("fileTransferMode")), FileTransferArchive(data.get("fileTransferArchive")))
+
+            return BaseService(
+                servicename=data["servicename"],
+                implements=data.get("implements"),
+                fileTransferMode=FileTransferMode(
+                    data.get("fileTransferMode", 0)),
+                fileTransferArchive=FileTransferArchive(
+                    data.get("fileTransferArchive", 0)),
+                description=data.get("description"),
+                icon=data.get("icon"),
+                infoUrl=data.get("infoUrl"),
+                helpUrl=data.get("helpUrl"),
+                displayName=data.get("displayName")
+            )
 
         raise ValueError("not a valid service json string.")
 
@@ -159,12 +244,18 @@ class BaseService:
 
         try:
             return BaseService(
-                serviceDict["servicename"],
-                serviceDict.get("implements", ["metadata"]),
-                FileTransferMode(serviceDict.get("fileTransferMode", 0)),
-                FileTransferArchive(serviceDict.get("fileTransferArchive", 0))
+                servicename=serviceDict["servicename"],
+                implements=serviceDict.get("implements", ["metadata"]),
+                fileTransferMode=FileTransferMode(
+                    serviceDict.get("fileTransferMode", 0)),
+                fileTransferArchive=FileTransferArchive(
+                    serviceDict.get("fileTransferArchive", 0)),
+                description=serviceDict.get("description"),
+                icon=serviceDict.get("icon"),
+                infoUrl=serviceDict.get("infoUrl"),
+                helpUrl=serviceDict.get("helpUrl"),
+                displayName=serviceDict.get("displayName")
             )
-
         except:
             raise ValueError("not a valid service dict")
 
@@ -175,24 +266,17 @@ class LoginService(BaseService):
 
     def __init__(
         self,
-        servicename: str,
-        implements: list = None,
-        fileTransferMode: FileTransferMode = FileTransferMode.active,
-        fileTransferArchive: FileTransferArchive = FileTransferArchive.none,
         userId: bool = True,
-        password: bool = True
+        password: bool = True,
+        *args, **kwargs
     ):
         """Initialize Service with username:password authentication.
 
         Args:
-            servicename (str): The name of the service, which will be registered. Must be unique.
-            implements (list, optional): Specified the implemented port endpoints. Defaults to empty list.
-            fileTransferMode (int, optional): Set the mode for transfering files. Defaults to 0=active. Alternative is 1=passive.
-            fileTransferArchive (str, optional): Set the archive, which is needed for transfering files. Defaults to "". Other value is "zip"
             userId (bool, optional): Set True, if username is needed to work. Defaults to True.
             password (bool, optional): Set True, if password is needed to work. Defaults to True.
         """
-        super().__init__(servicename, implements, fileTransferArchive, fileTransferArchive)
+        super().__init__(*args, **kwargs)
 
         self._userId = userId
         self._password = password
@@ -280,12 +364,17 @@ class LoginService(BaseService):
         password: bool
     ):
         return cls(
-            service.servicename,
-            service.implements,
-            service.fileTransferMode,
-            service.fileTransferArchive,
-            userId,
-            password
+            userId=userId,
+            password=password,
+            servicename=service.servicename,
+            implements=service.implements,
+            fileTransferMode=service.fileTransferMode,
+            fileTransferArchive=service.fileTransferArchive,
+            description=service.description,
+            icon=service.icon,
+            infoUrl=service.infoUrl,
+            helpUrl=service.helpUrl,
+            displayName=service.displayName,
         )
 
 
@@ -302,16 +391,21 @@ class OAuth2Service(BaseService):
 
     def __init__(
         self,
-        servicename: str = "",
-        implements: list = None,
-        fileTransferMode: FileTransferMode = FileTransferMode.active,
-        fileTransferArchive: FileTransferArchive = FileTransferArchive.none,
         authorize_url: str = "",
         refresh_url: str = "",
         client_id: str = "",
-        client_secret: str = ""
+        client_secret: str = "",
+        *args, **kwargs
     ):
-        super().__init__(servicename, implements, fileTransferArchive, fileTransferArchive)
+        """Initialize a service for oauth2.
+
+        Args:
+            authorize_url (str, optional): The authorize url from oauth2 workflow. Defaults to "".
+            refresh_url (str, optional): The refresh url from oauth2 workflow. Defaults to "".
+            client_id (str, optional): The client id from oauth2 workflow. Defaults to "".
+            client_secret (str, optional): The client secret from oauth2 workflow. Defaults to "".
+        """
+        super().__init__(*args, **kwargs)
 
         self.check_string(authorize_url, "authorize_url")
         self.check_string(refresh_url, "refresh_url")
@@ -462,12 +556,19 @@ class OAuth2Service(BaseService):
         Converts the given Service to an oauth2service.
         """
         return cls(
-            service.servicename,
-            service.implements, service.fileTransferMode, service.fileTransferArchive,
-            authorize_url,
-            refresh_url,
-            client_id,
-            client_secret,
+            authorize_url=authorize_url,
+            refresh_url=refresh_url,
+            client_id=client_id,
+            client_secret=client_secret,
+            servicename=service.servicename,
+            implements=service.implements,
+            fileTransferMode=service.fileTransferMode,
+            fileTransferArchive=service.fileTransferArchive,
+            description=service.description,
+            icon=service.icon,
+            infoUrl=service.infoUrl,
+            helpUrl=service.helpUrl,
+            displayName=service.displayName,
         )
 
     def __eq__(self, obj):
@@ -518,7 +619,7 @@ class OAuth2Service(BaseService):
                 data["authorize_url"],
                 data["refresh_url"],
                 data["client_id"],
-                data["client_secret"],
+                data.get("client_secret", ""),
             )
         except:
             raise ValueError("not a valid oauthservice json string.")
@@ -537,7 +638,7 @@ class OAuth2Service(BaseService):
                 serviceDict["authorize_url"],
                 serviceDict["refresh_url"],
                 serviceDict["client_id"],
-                serviceDict["client_secret"],
+                serviceDict.get("client_secret", ""),
             )
         except:
             raise ValueError("not a valid oauthservice dict.")
